@@ -18,7 +18,6 @@ import (
 	"k8s.io/api/node/v1beta1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/klog"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
@@ -28,6 +27,7 @@ import (
 
 	tunedv1 "github.com/openshift/cluster-node-tuning-operator/pkg/apis/tuned/v1"
 	machineconfigv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
+	mcov1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 
 	performancev1 "github.com/openshift-kni/performance-addon-operators/api/v1"
 	performancev1alpha1 "github.com/openshift-kni/performance-addon-operators/api/v1alpha1"
@@ -43,7 +43,7 @@ import (
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/profiles"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/machineconfig"
-	profilecomponent "github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/profile"
+	componentprofile "github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/profile"
 )
 
 const (
@@ -54,7 +54,6 @@ const (
 var RunningOnSingleNode bool
 
 var _ = Describe("[rfe_id:27368][performance]", func() {
-
 	var workerRTNodes []corev1.Node
 	var profile *performancev2.PerformanceProfile
 
@@ -133,7 +132,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 		It("[test_id:37127] Node should point to right tuned profile", func() {
 			for _, node := range workerRTNodes {
 				tuned := nodes.TunedForNode(&node, RunningOnSingleNode)
-				activeProfile, err := pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
+				activeProfile, err := pods.WaitForPodOutput(testclient.K8sClient, tuned, []string{"cat", "/etc/tuned/active_profile"})
 				Expect(err).ToNot(HaveOccurred(), "Error getting the tuned active profile")
 				activeProfileName := string(activeProfile)
 				Expect(strings.TrimSpace(activeProfileName)).To(Equal(tunedExpectedName), "active profile name mismatch got %q expected %q", activeProfileName, tunedExpectedName)
@@ -166,37 +165,49 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 
 		It("[test_id:27081][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] Should set workqueue CPU mask", func() {
 			for _, node := range workerRTNodes {
-				By("Getting tuned.non_isolcpus kernel argument")
+				By(fmt.Sprintf("Getting tuned.non_isolcpus kernel argument on %q", node.Name))
 				cmdline, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/proc/cmdline"})
+				Expect(err).ToNot(HaveOccurred())
 				re := regexp.MustCompile(`tuned.non_isolcpus=\S+`)
 				nonIsolcpusFullArgument := re.FindString(string(cmdline))
-				Expect(nonIsolcpusFullArgument).To(ContainSubstring("tuned.non_isolcpus="))
+				Expect(nonIsolcpusFullArgument).To(ContainSubstring("tuned.non_isolcpus="), "tuned.non_isolcpus parameter not found in %q", cmdline)
 				nonIsolcpusMask := strings.Split(string(nonIsolcpusFullArgument), "=")[1]
 				nonIsolcpusMaskNoDelimiters := strings.Replace(nonIsolcpusMask, ",", "", -1)
+
+				getTrimmedMaskFromData := func(maskType string, data []byte) string {
+					trimmed := strings.TrimSpace(string(data))
+					testlog.Infof("workqueue %s mask for %q: %q", maskType, node.Name, trimmed)
+					return strings.Replace(trimmed, ",", "", -1)
+				}
+
+				expectMasksEqual := func(expected, got string) {
+					expectedTrimmed := strings.TrimLeft(expected, "0")
+					gotTrimmed := strings.TrimLeft(got, "0")
+					ExpectWithOffset(1, expectedTrimmed).Should(Equal(gotTrimmed), "wrong workqueue mask on %q - got %q (from %q) expected %q (from %q)", node.Name, expectedTrimmed, expected, got, gotTrimmed)
+				}
+
+				By(fmt.Sprintf("Getting the virtual workqueue mask (/sys/devices/virtual/workqueue/cpumask) on %q", node.Name))
+				workqueueMaskData, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/sys/devices/virtual/workqueue/cpumask"})
 				Expect(err).ToNot(HaveOccurred())
-				By("executing the command \"cat /sys/devices/virtual/workqueue/cpumask\"")
-				workqueueMask, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/sys/devices/virtual/workqueue/cpumask"})
+				workqueueMask := getTrimmedMaskFromData("virtual", workqueueMaskData)
+				expectMasksEqual(nonIsolcpusMaskNoDelimiters, workqueueMask)
+
+				By(fmt.Sprintf("Getting the writeback workqueue mask (/sys/bus/workqueue/devices/writeback/cpumask) on %q", node.Name))
+				workqueueWritebackMaskData, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/sys/bus/workqueue/devices/writeback/cpumask"})
 				Expect(err).ToNot(HaveOccurred())
-				workqueueMaskTrimmed := strings.TrimSpace(string(workqueueMask))
-				workqueueMaskTrimmedNoDelimiters := strings.Replace(workqueueMaskTrimmed, ",", "", -1)
-				Expect(strings.TrimLeft(nonIsolcpusMaskNoDelimiters, "0")).Should(Equal(strings.TrimLeft(workqueueMaskTrimmedNoDelimiters, "0")), "workqueueMask is not set to "+workqueueMaskTrimmed)
-				By("executing the command \"cat /sys/bus/workqueue/devices/writeback/cpumask\"")
-				workqueueWritebackMask, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/sys/bus/workqueue/devices/writeback/cpumask"})
-				Expect(err).ToNot(HaveOccurred())
-				workqueueWritebackMaskTrimmed := strings.TrimSpace(string(workqueueWritebackMask))
-				workqueueWritebackMaskTrimmedNoDelimiters := strings.Replace(workqueueWritebackMaskTrimmed, ",", "", -1)
-				Expect(strings.TrimLeft(nonIsolcpusMaskNoDelimiters, "0")).Should(Equal(strings.TrimLeft(workqueueWritebackMaskTrimmedNoDelimiters, "0")), "workqueueMask is not set to "+workqueueWritebackMaskTrimmed)
+				workqueueWritebackMask := getTrimmedMaskFromData("workqueue", workqueueWritebackMaskData)
+				expectMasksEqual(nonIsolcpusMaskNoDelimiters, workqueueWritebackMask)
 			}
 		})
 
 		It("[test_id:32375][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] initramfs should not have injected configuration", func() {
-			Skip("Skipping test until BZ#1858347 is resolved")
 			for _, node := range workerRTNodes {
 				rhcosId, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"awk", "-F", "/", "{printf $3}", "/rootfs/proc/cmdline"})
 				Expect(err).ToNot(HaveOccurred())
 				initramfsImagesPath, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"find", filepath.Join("/rootfs/boot/ostree", string(rhcosId)), "-name", "*.img"})
 				Expect(err).ToNot(HaveOccurred())
-				initrd, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"lsinitrd", strings.TrimSpace(string(initramfsImagesPath))})
+				modifiedImagePath := strings.TrimPrefix(strings.TrimSpace(string(initramfsImagesPath)), "/rootfs")
+				initrd, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"chroot", "/rootfs", "lsinitrd", modifiedImagePath})
 				Expect(err).ToNot(HaveOccurred())
 				Expect(string(initrd)).ShouldNot(ContainSubstring("'/etc/systemd/system.conf /etc/systemd/system.conf.d/setAffinity.conf'"))
 			}
@@ -205,7 +216,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 		It("[test_id:35363][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] stalld daemon is running on the host", func() {
 			for _, node := range workerRTNodes {
 				tuned := nodes.TunedForNode(&node, RunningOnSingleNode)
-				_, err := pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
+				_, err := pods.WaitForPodOutput(testclient.K8sClient, tuned, []string{"pidof", "stalld"})
 				Expect(err).ToNot(HaveOccurred())
 			}
 		})
@@ -360,7 +371,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 
 				for _, pod := range nodePods.Items {
 					cmd := []string{"find", "/sys/devices", "-type", "f", "-name", "rps_cpus", "-exec", "cat", "{}", ";"}
-					devsRPS, err := pods.ExecCommandOnPod(&pod, cmd)
+					devsRPS, err := pods.WaitForPodOutput(testclient.K8sClient, &pod, cmd)
 					for _, devRPS := range strings.Split(strings.Trim(string(devsRPS), "\n"), "\n") {
 						rpsCPUs, err = components.CPUMaskToCPUSet(devRPS)
 						Expect(err).ToNot(HaveOccurred())
@@ -394,7 +405,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 	})
 
 	Context("KubeletConfig experimental annotation", func() {
-		var secondMCP *machineconfigv1.MachineConfigPool
+		var secondMCP *mcov1.MachineConfigPool
 		var secondProfile *performancev2.PerformanceProfile
 		var newRole = "test-annotation"
 
@@ -431,15 +442,15 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 			Expect(testclient.Client.Create(context.TODO(), secondProfile)).ToNot(HaveOccurred())
 
-			machineConfigSelector := profilecomponent.GetMachineConfigLabel(secondProfile)
-			secondMCP = &machineconfigv1.MachineConfigPool{
+			machineConfigSelector := componentprofile.GetMachineConfigLabel(secondProfile)
+			secondMCP = &mcov1.MachineConfigPool{
 				ObjectMeta: metav1.ObjectMeta{
 					Name: "test-annotation",
 					Labels: map[string]string{
 						machineconfigv1.MachineConfigRoleLabelKey: newRole,
 					},
 				},
-				Spec: machineconfigv1.MachineConfigPoolSpec{
+				Spec: mcov1.MachineConfigPoolSpec{
 					MachineConfigSelector: &metav1.LabelSelector{
 						MatchLabels: machineConfigSelector,
 					},
@@ -479,13 +490,13 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 
 		AfterEach(func() {
 			if secondProfile != nil {
-				if err := testclient.Client.Delete(context.TODO(), secondProfile); err != nil {
+				if err := profiles.Delete(secondProfile.Name); err != nil {
 					klog.Warningf("failed to delete the performance profile %q: %v", secondProfile.Name, err)
 				}
 			}
 
 			if secondMCP != nil {
-				if err := testclient.Client.Delete(context.TODO(), secondMCP); err != nil {
+				if err := mcps.Delete(secondMCP.Name); err != nil {
 					klog.Warningf("failed to delete the machine config pool %q: %v", secondMCP.Name, err)
 				}
 			}
@@ -493,20 +504,23 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 	})
 
 	Context("Create second performance profiles on a cluster", func() {
-		It("[test_id:32364] Verifies that cluster can have multiple profiles", func() {
-			newRole := "worker-new"
+		var secondMCP *mcov1.MachineConfigPool
+		var secondProfile *performancev2.PerformanceProfile
+		var newRole = "worker-new"
+
+		BeforeEach(func() {
 			newLabel := fmt.Sprintf("%s/%s", testutils.LabelRole, newRole)
 
 			reserved := performancev2.CPUSet("0")
 			isolated := performancev2.CPUSet("1-3")
 
-			secondProfile := &performancev2.PerformanceProfile{
+			secondProfile = &performancev2.PerformanceProfile{
 				TypeMeta: metav1.TypeMeta{
 					Kind:       "PerformanceProfile",
 					APIVersion: performancev2.GroupVersion.String(),
 				},
 				ObjectMeta: metav1.ObjectMeta{
-					Name: "profile2",
+					Name: "second-profile",
 				},
 				Spec: performancev2.PerformanceProfileSpec{
 					CPU: &performancev2.CPU{
@@ -525,6 +539,45 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 					},
 				},
 			}
+
+			machineConfigSelector := componentprofile.GetMachineConfigLabel(secondProfile)
+			secondMCP = &mcov1.MachineConfigPool{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "second-mcp",
+					Labels: map[string]string{
+						machineconfigv1.MachineConfigRoleLabelKey: newRole,
+					},
+				},
+				Spec: mcov1.MachineConfigPoolSpec{
+					MachineConfigSelector: &metav1.LabelSelector{
+						MatchLabels: machineConfigSelector,
+					},
+					NodeSelector: &metav1.LabelSelector{
+						MatchLabels: map[string]string{
+							newLabel: "",
+						},
+					},
+				},
+			}
+
+			Expect(testclient.Client.Create(context.TODO(), secondMCP)).ToNot(HaveOccurred())
+		})
+
+		AfterEach(func() {
+			if secondProfile != nil {
+				if err := profiles.Delete(secondProfile.Name); err != nil {
+					klog.Warningf("failed to delete the performance profile %q: %v", secondProfile.Name, err)
+				}
+			}
+
+			if secondMCP != nil {
+				if err := mcps.Delete(secondMCP.Name); err != nil {
+					klog.Warningf("failed to delete the machine config pool %q: %v", secondMCP.Name, err)
+				}
+			}
+		})
+
+		It("[test_id:32364] Verifies that cluster can have multiple profiles", func() {
 			Expect(testclient.Client.Create(context.TODO(), secondProfile)).ToNot(HaveOccurred())
 
 			By("Checking that new KubeletConfig, MachineConfig and RuntimeClass created")
@@ -624,16 +677,18 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			v1Profile.Spec.MachineConfigLabel = nil
 			Expect(testclient.Client.Create(context.TODO(), v1Profile)).ToNot(HaveOccurred())
 
-			key = types.NamespacedName{
-				Name:      v1Profile.Name,
-				Namespace: v1Profile.Namespace,
-			}
-
 			defer func() {
 				Expect(testclient.Client.Delete(context.TODO(), v1Profile)).ToNot(HaveOccurred())
 				Expect(profiles.WaitForDeletion(key, 60*time.Second)).ToNot(HaveOccurred())
 			}()
 
+			key = types.NamespacedName{
+				Name:      v1Profile.Name,
+				Namespace: v1Profile.Namespace,
+			}
+			err = testclient.Client.Get(context.TODO(), key, v1Profile)
+			Expect(err).ToNot(HaveOccurred(), "Failed getting v1Profile")
+
 			v2Profile := &performancev2.PerformanceProfile{}
 			err = testclient.GetWithRetry(context.TODO(), key, v2Profile)
 			Expect(err).ToNot(HaveOccurred(), "Failed getting v2Profile")
@@ -686,6 +741,70 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			Expect(verifyV1alpha1Conversion(v1alpha1Profile, v1Profile)).ToNot(HaveOccurred())
 		}
 
+		// empty context to use the same JustBeforeEach and AfterEach
+		Context("", func() {
+			var testProfileName string
+			var globallyDisableIrqLoadBalancing bool
+
+			JustBeforeEach(func() {
+				key := types.NamespacedName{
+					Name:      profile.Name,
+					Namespace: profile.Namespace,
+				}
+				err := testclient.Client.Get(context.TODO(), key, profile)
+				Expect(err).ToNot(HaveOccurred(), "Failed to get profile")
+
+				profile.Name = testProfileName
+				profile.ResourceVersion = ""
+				profile.Spec.NodeSelector = map[string]string{"test/test": "test"}
+				profile.Spec.GloballyDisableIrqLoadBalancing = pointer.BoolPtr(globallyDisableIrqLoadBalancing)
+				profile.Spec.MachineConfigPoolSelector = nil
+				profile.Spec.MachineConfigLabel = nil
+
+				err = testclient.Client.Create(context.TODO(), profile)
+				Expect(err).ToNot(HaveOccurred(), "Failed to create profile")
+
+				// we need to get updated profile object after the name and spec changes
+				key = types.NamespacedName{
+					Name:      profile.Name,
+					Namespace: profile.Namespace,
+				}
+				err = testclient.Client.Get(context.TODO(), key, profile)
+				Expect(err).ToNot(HaveOccurred(), "Failed to get profile")
+			})
+
+			When("the GloballyDisableIrqLoadBalancing field set to false", func() {
+				BeforeEach(func() {
+					testProfileName = "gdilb-false"
+					globallyDisableIrqLoadBalancing = false
+				})
+
+				It("should preserve the value during the v1 <-> v2 conversion", func() {
+					verifyV2V1()
+				})
+			})
+
+			When("the GloballyDisableIrqLoadBalancing field set to true", func() {
+				BeforeEach(func() {
+					testProfileName = "gdilb-true"
+					globallyDisableIrqLoadBalancing = true
+				})
+
+				It("should preserve the value during the v1 <-> v2 conversion", func() {
+					verifyV2V1()
+				})
+			})
+
+			AfterEach(func() {
+				Expect(testclient.Client.Delete(context.TODO(), profile)).ToNot(HaveOccurred())
+				Expect(profiles.WaitForDeletion(types.NamespacedName{
+					Name:      profile.Name,
+					Namespace: profile.Namespace,
+				}, 60*time.Second)).ToNot(HaveOccurred())
+			})
+
+		})
+
 		When("the performance profile does not contain NUMA field", func() {
 			BeforeEach(func() {
 				key := types.NamespacedName{
@@ -739,7 +858,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 		})
 
-		validateObject := func(obj runtime.Object, message string) {
+		validateObject := func(obj client.Object, message string) {
 			err := testclient.Client.Create(context.TODO(), obj)
 			Expect(err).To(HaveOccurred(), "expected the validation error")
 			Expect(err.Error()).To(ContainSubstring(message))
@@ -1182,7 +1301,7 @@ func verifyV2Conversion(v2Profile *performancev2.PerformanceProfile, v1Profile *
 	for _, f := range v2Profile.GetObjectMeta().GetManagedFields() {
 		if f.APIVersion == performancev1alpha1.GroupVersion.String() ||
 			f.APIVersion == performancev1.GroupVersion.String() {
-			if v2Profile.Spec.GloballyDisableIrqLoadBalancing == nil || !*v2Profile.Spec.GloballyDisableIrqLoadBalancing {
+			if v2Profile.Spec.GloballyDisableIrqLoadBalancing == nil {
 				return fmt.Errorf("globallyDisableIrqLoadBalancing field must be set to true")
 			}
 		}
@@ -1229,7 +1348,7 @@ func validateTunedActiveProfile(wrknodes []corev1.Node) {
 		tunedName := tuned.ObjectMeta.Name
 		By(fmt.Sprintf("executing the command cat /etc/tuned/active_profile inside the pod %s", tunedName))
 		Eventually(func() string {
-			out, err = pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
+			out, err = pods.WaitForPodOutput(testclient.K8sClient, tuned, []string{"cat", "/etc/tuned/active_profile"})
 			return strings.TrimSpace(string(out))
 		}, cluster.ComputeTestTimeout(testTimeout*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(Equal(activeProfileName),
 			fmt.Sprintf("active_profile is not set to %s. %v", activeProfileName, err))
