@@ -4,7 +4,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"github.com/ghodss/yaml"
 	"path"
 	"strconv"
 	"strings"
@@ -12,6 +11,8 @@ import (
 
 	. "github.com/onsi/gomega"
 
+	"github.com/ghodss/yaml"
+
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
@@ -24,6 +25,7 @@ import (
 	testclient "github.com/openshift-kni/performance-addon-operators/functests/utils/client"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/cluster"
 	testlog "github.com/openshift-kni/performance-addon-operators/functests/utils/log"
+	testpods "github.com/openshift-kni/performance-addon-operators/functests/utils/pods"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components"
 )
 
@@ -109,17 +111,7 @@ func ExecCommandOnMachineConfigDaemon(node *corev1.Node, command []string) ([]by
 	}
 	testlog.Infof("found mcd %s for node %s", mcd.Name, node.Name)
 
-	initialArgs := []string{
-		"exec",
-		"-i",
-		"-n", testutils.NamespaceMachineConfigOperator,
-		"-c", testutils.ContainerMachineConfigDaemon,
-		"--request-timeout", "30",
-		mcd.Name,
-		"--",
-	}
-	initialArgs = append(initialArgs, command...)
-	return testutils.ExecAndLogCommand("oc", initialArgs...)
+	return testpods.WaitForPodOutput(testclient.K8sClient, mcd, command)
 }
 
 // ExecCommandOnNode executes given command on given node and returns the result
@@ -128,7 +120,9 @@ func ExecCommandOnNode(cmd []string, node *corev1.Node) (string, error) {
 	if err != nil {
 		return "", err
 	}
-	return strings.Trim(string(out), "\n"), nil
+
+	trimmedString := strings.Trim(string(out), "\n")
+	return strings.ReplaceAll(trimmedString, "\r", ""), nil
 }
 
 // GetKubeletConfig returns KubeletConfiguration loaded from the node /etc/kubernetes/kubelet.conf
