@@ -5,13 +5,13 @@ import (
 	"time"
 
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	kubeletconfigv1beta1 "k8s.io/kubelet/config/v1beta1"
 
 	performancev2 "github.com/openshift-kni/performance-addon-operators/api/v2"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components"
-	profile2 "github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/profile"
 	machineconfigv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 )
 
@@ -28,6 +28,7 @@ const (
 	// Please avoid specifying them and use the relevant API to configure these parameters.
 	experimentalKubeletSnippetAnnotation = "kubeletconfig.experimental"
 	cpuManagerPolicyStatic               = "static"
+	memoryManagerPolicyStatic            = "Static"
 	defaultKubeReservedMemory            = "500Mi"
 	defaultSystemReservedMemory          = "500Mi"
 	defaultHardEvictionThreshold         = "100Mi"
@@ -35,7 +36,7 @@ const (
 )
 
 // New returns new KubeletConfig object for performance sensetive workflows
-func New(profile *performancev2.PerformanceProfile) (*machineconfigv1.KubeletConfig, error) {
+func New(profile *performancev2.PerformanceProfile, profileMCPLabels map[string]string) (*machineconfigv1.KubeletConfig, error) {
 	name := components.GetComponentName(profile.Name, components.ComponentNamePrefix)
 	kubeletConfig := &kubeletconfigv1beta1.KubeletConfiguration{}
 	if v, ok := profile.Annotations[experimentalKubeletSnippetAnnotation]; ok {
@@ -83,7 +84,39 @@ func New(profile *performancev2.PerformanceProfile) (*machineconfigv1.KubeletCon
 
 	if profile.Spec.NUMA != nil {
 		if profile.Spec.NUMA.TopologyPolicy != nil {
-			kubeletConfig.TopologyManagerPolicy = *profile.Spec.NUMA.TopologyPolicy
+			topologyPolicy := *profile.Spec.NUMA.TopologyPolicy
+			kubeletConfig.TopologyManagerPolicy = topologyPolicy
+
+			// set the memory manager policy to static only when the topology policy is
+			// restricted or single NUMA node
+			if topologyPolicy == kubeletconfigv1beta1.RestrictedTopologyManagerPolicy ||
+				topologyPolicy == kubeletconfigv1beta1.SingleNumaNodeTopologyManagerPolicy {
+				kubeletConfig.MemoryManagerPolicy = memoryManagerPolicyStatic
+
+				if kubeletConfig.ReservedMemory == nil {
+					reservedMemory := resource.NewQuantity(0, resource.DecimalSI)
+					if err := addStringToQuantity(reservedMemory, kubeletConfig.KubeReserved[string(corev1.ResourceMemory)]); err != nil {
+						return nil, err
+					}
+					if err := addStringToQuantity(reservedMemory, kubeletConfig.SystemReserved[string(corev1.ResourceMemory)]); err != nil {
+						return nil, err
+					}
+					if err := addStringToQuantity(reservedMemory, kubeletConfig.EvictionHard[evictionHardMemoryAvailable]); err != nil {
+						return nil, err
+					}
+
+					kubeletConfig.ReservedMemory = []kubeletconfigv1beta1.MemoryReservation{
+						{
+							// the NUMA node 0 is the only safe choice for non NUMA machines
+							//  in the future we can extend our API to get this information from a user
+							NumaNode: 0,
+							Limits: map[corev1.ResourceName]resource.Quantity{
+								corev1.ResourceMemory: *reservedMemory,
+							},
+						},
+					}
+				}
+			}
 		}
 	}
 
@@ -102,7 +135,7 @@ func New(profile *performancev2.PerformanceProfile) (*machineconfigv1.KubeletCon
 		},
 		Spec: machineconfigv1.KubeletConfigSpec{
 			MachineConfigPoolSelector: &metav1.LabelSelector{
-				MatchLabels: profile2.GetMachineConfigPoolSelector(profile),
+				MatchLabels: profileMCPLabels,
 			},
 			KubeletConfig: &runtime.RawExtension{
 				Raw: raw,
@@ -110,3 +143,13 @@ func New(profile *performancev2.PerformanceProfile) (*machineconfigv1.KubeletCon
 		},
 	}, nil
 }
+
+func addStringToQuantity(q *resource.Quantity, value string) error {
+	v, err := resource.ParseQuantity(value)
+	if err != nil {
+		return err
+	}
+	q.Add(v)
+
+	return nil
+}
